
#include <math.h>
//Windows header
#ifdef WIN32
#include <windows.h>
#endif

// GLEW header for OpenGL extentions loading
// OpenGL header
#include <GL/gl.h>
// Glut for display window managment
#include <gl/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>

using namespace std;
#define M_PI 3.14159265358979323846


#include "vec.h"
#include "dynamics.h"


/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 3.0f;

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);

		glLoadIdentity();
		gluLookAt(x, y, z, -x, y, -z,	0.0f,1.0f,0.0f);
}




/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0;
Dynamics dynamics;

Vec initPos (0.0, 5.0, 0.0);
int nbBalls = 2;
float mass = 1.0;
float radius = 0.25;

float stiffness = 30.0;
float initLength = 0.2;
float damping = 0.2;

void updateAnimationParameters(){
		// compute elapsed time
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float dt = (newTime-oldTime);
		oldTime = newTime;

		dynamics.animate(dt);
}

void setDynamics(){
		// add a fixed ball
		int ball0 = dynamics.addBall(initPos, Vec(), 0.0, radius);

		// add a second ball
		Vec position  = initPos + Vec(0.0, -2.0*radius, 0.0);
		int ball1 = dynamics.addBall(position, Vec(1.0, 0.0, 0.0), mass, radius);
}


////////////////////////////////////////
// Initialization methods
////////////////////////////////////////



void initScene() {
		// init GL states
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);				
		glClearDepth(1.0);													
		glDepthFunc(GL_LESS);												
		glEnable(GL_DEPTH_TEST);										
		glShadeModel(GL_SMOOTH);										
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		// init dynamics
		setDynamics();
}




void changeSize(int w, int h)   {

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;

		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);

		// Set the clipping volume
		gluPerspective(45,ratio,1,1000);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}







///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawGround(){

		float backColor[] = {1.0, 0.0, 0.0};
		float frontColor[] = {0.0, 0.0, 1.0};
		
		glBegin(GL_QUADS);
		
		glColor3fv(frontColor);
		glVertex3f(-10.0f, 0.0f, -10.0f);
		glColor3fv(backColor);
		glVertex3f(-10.0f, 0.0f,  10.0f);
		glColor3fv(backColor);
		glVertex3f( 10.0f, 0.0f,  10.0f);
		glColor3fv(frontColor);
		glVertex3f( 10.0f, 0.0f, -10.0f);
		
		glEnd();
}



void drawSphere(){
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				float intensity1 = (float)i/nbStrips;
				float intensity2 = (float)(i+1)/nbStrips;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						
						glNormal3fv(pos1);
						glColor3f(intensity1, 0.0, 1.0-intensity1);
						glVertex3fv(pos1);
						
						glNormal3fv(pos2);
						glColor3f(intensity2, 0.0, 1.0-intensity2);
						glVertex3fv(pos2);
				}
				glEnd();
		}
}


void drawBalls(){
		for (int i=0 ; i<nbBalls ; i++){
				glPushMatrix();
				
				Vec pos = dynamics.getBallPos(i);
				glTranslatef(pos.x, pos.y, pos.z);

				double radius = dynamics.getBallRadius(i);
				glScaled(radius, radius, radius);
				
				drawSphere();
				
				glPopMatrix();
		}
}

void drawSprings(){
		int i = 0;
		int nbSprings = dynamics.getNbSprings();
		glBegin(GL_LINES);
		for (int i=0 ; i<nbSprings ; i++){
				int obj1 = dynamics.getSpring(i).obj1;
				int obj2 = dynamics.getSpring(i).obj2;
				Vec pos1 = dynamics.getBallPos(obj1);
				Vec pos2 = dynamics.getBallPos(obj2);
				glColor3f(1.0, 1.0, 1.0);
				glVertex3f(pos1.x, pos1.y, pos1.z);
				glVertex3f(pos2.x, pos2.y, pos2.z);
		}		
		glEnd();
}


void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		drawGround();
		drawBalls();
		drawSprings();

		// updateAnimationParameters
		updateAnimationParameters();
								
		// display rendering
		glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////

int prevX = 0, prevY = 0;
bool dragging = false;
float normDist = 1000.0f;

void specialKey(int key, int x_, int y_) {

		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.1f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.1f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.2f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.2f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
		switch (key) {
		default:
				cout << key << endl; break;
		}
}



void mouseClick(int button, int state, int x,int y) {
		if (button == GLUT_LEFT_BUTTON && state==GLUT_DOWN){
				dragging = true;
				prevX = x;
				prevY = y;
		} else if (button == GLUT_LEFT_BUTTON && state==GLUT_UP){
				dragging = false;				
		}
		
}

void mouseMotion(int x,int y) {
	if (dragging) {
		float dx = (x-prevX)*distToOrigin/normDist;
		float dy = (y-prevY)*distToOrigin/normDist;

		initPos.x += sin(cameraAngle)*dx;
		initPos.z -=  cos(cameraAngle)*dx;
		initPos.y -= dy;
				
		dynamics.setBallPos(0,initPos);

		prevX = x;
		prevY = y;
	}
}


////////////////////////////////////
//  Main method
////////////////////////////////////

int main(int argc, char **argv) {

		// create window
		glutInit(&argc, argv);
		glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
		glutInitWindowPosition(100,100);
		glutInitWindowSize(800, 600);
		glutCreateWindow("Dynamics");

		// initialize OpenGL states
		initScene();

		// Bind Key events
		glutSpecialFunc(specialKey);
		glutKeyboardFunc(commonKey);
		glutMouseFunc(mouseClick);
		glutMotionFunc(mouseMotion);
		
		// Bind rendering methods
		glutDisplayFunc(renderScene);
		glutIdleFunc(renderScene);

		glutReshapeFunc(changeSize);

		// run program
		glutMainLoop();

		return(0);
}
